#ifndef DIALOG_H
#define DIALOG_H

#include <QDebug>
#include <QtWidgets/QDialog>
#include <QPainter>
#include <QtWidgets/QMessageBox>
#include <QDir>
#include <QTextStream>
#include <QTimer>
#include <QGraphicsPixmapItem>
#include <QtSerialPort/QtSerialPort>
#include <QtMultimedia/QSound>
#include <rf4ceprotocolhandler.h>
#include <rf4ceaudiohandler.h>
#include <serialhandler.h>

#define APP_TITLE					"RF4CE Remote Demo"

#define REMOTE_INI_FILE             "remote.ini"
#define REMOTE_MAIN_IMAGE_FILE      "remote.bmp"
#define REMOTE_PRESSED_IMAGE_FILE   "remotePressed.bmp"
#define REMOTE_RELEASED_IMAGE_FILE  "remoteReleased.bmp"
#define TEST_TIME_MULTIPLIER        4

#define CUSTOM_CODEC_STRING "Custom INI Codec Settings"

struct KeyProfile : public QObject
{
    KeyProfile(QObject*){}
    QString name;
    int left, top, right, bottom;
    uint code;
};

struct RemoteProfile : public QObject
{
    RemoteProfile(QObject*){}
    QString profileName;
    QImage remoteImage, remoteImagePressed, remoteImageReleased;
    QPixmap remotePixmap, remotePixmapPressed, remotePixmapReleased;
    int totalKeyCount;
    int keyTestTime;
    QVector<KeyProfile*> keys;
};

namespace Ui {
class Dialog;
}

class Dialog : public QDialog
{
    Q_OBJECT

public:
    explicit Dialog(QWidget *parent = 0);
    ~Dialog();

public slots:
    void keyCodePressed(int keyCode, unsigned char keyFlags);
    void postTrace(QString traceString);
    void addWaveFile(QString waveFileName);

private slots:
    void on_simulateButton_clicked();
    void on_profileComboBox_currentIndexChanged(int index);
    void on_timeout();
    void on_refreshPortsButton_clicked();
    void on_clearTraceButton_clicked();
    void on_waveFileListWidget_doubleClicked(const QModelIndex &index);
    void on_playSelectedAudioPushButton_clicked();
    void on_clearAudioPushButton_clicked();
    void on_comPortComboBox_currentIndexChanged(int index);
    void on_saveTraceDataPushButton_clicked();
    void on_realtimeAudioCheckBox_clicked();
    void on_quickSetButton_clicked();
    void on_pairButton_clicked();
    void on_identifyButton_clicked();
    void on_codecTypeComboBox_currentIndexChanged(const QString &arg1);

private:
    Ui::Dialog *ui;
    QAbstractItemModel* waveFileList;

    // Rename to RF4CESerialHandler
    SerialHandler* serialHandler;
    RF4CEAudioHandler* audioHandler;
    RF4CEProtocolHandler* protocolHandler;

    QList<QSerialPortInfo> serialPortInfoList;
    QVector<RemoteProfile*> remotes;
    int currentProfile;
    bool connected;
    bool targetHeartbeat;
    bool controllerHeartbeat;
    int currentKeyCode, currentKeyIndex;
    bool pressingKey;
    bool releasingKey;
    bool testResult;

    // Custom audio settings from INI
    bool customIniAudioSettings;
    DecodeType audioDecodeType;
    int audioCompressedBufferSize;
    int16s offset;
    int16s gain;

    void paintEvent(QPaintEvent *);
    void closeEvent(QCloseEvent *);

    void refreshPorts();
    void setupAudioCodecOptions();
    void gatherProfiles();
    bool readIniFile(RemoteProfile* rp, QString iniFilePath);
    void resetTest();
    void playAudioFile(QString fileName);
};

#endif // DIALOG_H
