#ifndef RF4CEPROTOCOLHANDLER_H
#define RF4CEPROTOCOLHANDLER_H

#include <QDebug>
#include <QThread>
#include <QQueue>
#include <QMutex>
#include <QObject>
#include <rf4ceaudiohandler.h>
#include <serialhandler.h>

class SerialHandler;

#define MESSAGE_FRAME_START 0xC0
#define MESSAGE_FRAME_END 0xC1

#define MESSAGE_HEADER_VERSION_INDEX 0
#define MESSAGE_HEADER_TYPE_INDEX 1
#define MESSAGE_HEADER_LENGTH_INDEX 2

#define MESSAGE_KEYCODE_PAYLOAD_OFFSET 3

#define MESSAGE_HEADER_SIZE 3

#define MESSAGE_VERSION_CURRENT (char)0x00
#define MESSAGE_VERSION_UNDEFINED (char)0xFF
#define MESSAGE_ESCAPE_CHARACTER (char)0x7E
#define MESSAGE_ESCAPE_CHARACTER_MASK (char)0x20

#define MAX_PAYLOAD_LENGTH 255
#define APP_BUFF_SIZE (1 + 1 + 1 + MAX_PAYLOAD_LENGTH)
#define PRE_BUFF_SIZE (1 + 2 * (APP_BUFF_SIZE + 1) + 1)

enum MessageType
{
    GetStatusReq = 0,
    GetStatusAck = 1,
    Action = 10,
    ActionMapReq = 14,
    ActionMapAck = 15,
    Audio = 20,
    IdentifyReq = 40,
    IdentifyAck = 41,
    BindReq = 50,
    BindAck = 53,
    Undefined = -1,
};

class RF4CEProtocolHandler : public QThread
{
    Q_OBJECT

public:
    RF4CEProtocolHandler(QObject *parent=0, RF4CEAudioHandler* pAudioHandler=0);
    ~RF4CEProtocolHandler();

    void startStream();
    void stopStream();
    void enqueuePacket(QList<char>* packet);
    void sendIdentifyPacket(SerialHandler* serialHandler);
    void sendPairingPacket(SerialHandler* serialHandler);
    void sendToTarget(SerialHandler* serialHandler, unsigned char uMid, unsigned char *buffer, unsigned char bufferSize);
    int appEncode(unsigned char uMid, unsigned char *buffer, unsigned char bufferSize);
    int preEncode(int iAppTxBufLen);
    void preEsc(int *iPreTxBufLen, unsigned char uDat);

protected:
    void run();

private:
    RF4CEAudioHandler* audioHandler;
    volatile bool runStream;
    QMutex mutex;
    QQueue<QList<char>*> packetQueue;

    unsigned char xAppTxBufDat[APP_BUFF_SIZE];
    unsigned char xPreTxBufDat[PRE_BUFF_SIZE];
};

#endif // RF4CEPROTOCOLHANDLER_H
