#include "serialhandler.h"

SerialHandler::SerialHandler(QObject *parent, RF4CEProtocolHandler* pProtocolHandler) : QThread(parent)
{
    this->setParent(parent);
    this->protocolHandler = pProtocolHandler;
}

SerialHandler::~SerialHandler()
{

}

bool SerialHandler::startStream(const QSerialPortInfo* serialPortInfo)
{
    serialPort.setPort(*serialPortInfo);
    serialPort.setBaudRate(QSerialPort::Baud115200);
    serialPort.setParity(QSerialPort::NoParity);
    serialPort.setDataBits(QSerialPort::Data8);
    serialPort.setStopBits(QSerialPort::OneStop);
    serialPort.setFlowControl(QSerialPort::NoFlowControl);

    // Reset framing
    needNewPacket = true;

    if (!serialPort.open(QSerialPort::ReadWrite))
    {
        return false;
    }

    this->start();
    return true;
}

void SerialHandler::stopStream()
{
    // Shut down the thread and wait for completion
    runStream = false;
    // Add timeout to robustify this
    while(this->isRunning());
    serialPort.close();
}

int SerialHandler::write(unsigned char* buffer, unsigned char bufferLength)
{
    if (serialPort.isOpen())
    {
        return (int)serialPort.write((const char*)buffer, bufferLength);
    }

    return 0;
}

void SerialHandler::run()
{
    int bytesToRead;
    unsigned char currentByte;
    unsigned char* currentByteArray;
    int currentByteArrayIndex = 1;

    runStream = true;
    while (runStream && serialPort.isOpen())
    {
        // Setup our buffer to read from
        bytesToRead = serialPort.bytesAvailable();
        if (bytesToRead == 0)
        {
            // No bytes
            msleep(1);
            continue;
        }
        else if (bytesToRead < 0)
        {
            // Should never happen!
            qDebug() << "Fatal Error! Should not be negative";
            msleep(1);
            continue;
        }
        currentByteArrayIndex = 0;
        currentByteArray = (unsigned char*)malloc(bytesToRead);
        if (!currentByteArray)
        {
            qDebug() << "Error allocating memory for serial byte array";
            continue;
        }
        if (serialPort.read((char*)currentByteArray, bytesToRead) != bytesToRead)
        {
            free(currentByteArray);
            // This should never happen
            continue;
        }

        // Parse through each byte and feed it to the protocol handler
        while (bytesToRead != 0)
        {
            currentByte = currentByteArray[currentByteArrayIndex];
            bytesToRead--;
            currentByteArrayIndex++;

            // If we are looking for a new packet, wait for the start frame message
            if (needNewPacket)
            {
                if (currentByte == MESSAGE_FRAME_START)
                {
                    packet = new QList<char>;
                    if (packet != NULL)
                    {
                        needNewPacket = false;
                    }
                    else
                    {
                        qDebug() << "Error creating packet\n";
                    }
                }
                else
                {
                    // Framing error, check next byte
                }
                continue;
            }

            if (currentByte != MESSAGE_FRAME_END)
            {
                packet->append(currentByte);
            }
            else
            {
                protocolHandler->enqueuePacket(packet);
                needNewPacket = true;
            }
        }
        if (currentByteArray)
        {
            free(currentByteArray);
        }
    }
}
